import java.util.*;

public class Block extends AttachableElement  {
   private static int id=0;  // Block identification number
   private final float mass;
   private Vector2D pos_t;     // current position at time t
   private Vector2D pos_tPlusDelta;  // next position in delta time in future
   private Vector2D speed_t;   // speed at time t
   private Vector2D speed_tPlusDelta;   // speed in delta time in future
   private Vector2D a_t;    // acceleration at time t
   private Vector2D a_tMinusDelta;  // acceleration delta time ago;
   private Vector<Spring> springs;  // Vector can grow, arrays cannot. 
   
   private Block(){   // nobody can create a block without state
     super(id++);
     mass=0;
   }
   public Block(float mass, Vector2D position, Vector2D speed){
      super(id++);
      this.mass = mass;
      pos_t = position;
      speed_t = speed;
      a_t = a_tMinusDelta = new Vector2D();
      springs = new Vector<Spring>();
   }
   public void attachSpring (Spring spring) {
     springs.add(spring);
   }
   public void detachSpring (Spring spring) {
     springs.remove(spring);
   }
   public Vector2D getPosition() {
      return pos_t;
   }
   private Vector2D getNetForce(float g, float v_f_const) {
     Vector2D force = new Vector2D();
	  Vector2D gravity = new Vector2D(0,-mass*g);
	  Vector2D friction = new Vector2D();
	  for (Spring s: springs)
        force = force.add(s.getForce(this));		  
	  force = force.add(gravity);
	  friction = speed_t.multiply(v_f_const);
	  force = force.subtract(friction);
     return(force);    
   }
   public void setInitialState(float g, float v_f_const) {
      a_tMinusDelta = getNetForce(g,v_f_const).multiply(1/mass);
   }
   public void computeNextState(double delta_t, float g, float v_f_const) {
     a_t = getNetForce(g, v_f_const).multiply(1/mass);
     speed_tPlusDelta = speed_t.add( ((a_t.multiply(3)).subtract(a_tMinusDelta)).multiply(delta_t/2));
     pos_tPlusDelta = pos_t.add( (speed_t.multiply(delta_t)).add(((a_t.multiply(4)).subtract(a_tMinusDelta)).multiply(delta_t*delta_t/6)));
   }
   public void updateState(){
     pos_t = pos_tPlusDelta;
     speed_t = speed_tPlusDelta;
     a_tMinusDelta = a_t;
   }
   public String getDescription() {
      return "Block_" + getId()+": (x,y)";
   }
   public String getState() {
      return getPosition()+"";
   }
}
