import java.io.*;
import java.util.*;

public class MyWorld {
   public MyWorld(){
      g = 0;
		v_f_const = 0; 
      out = System.out;
      elements = new Vector<PhysicsElement>();
   }
   public void setGravity (float gravity) {
      g = gravity;
   }
	public void setViscousFrictionConstant (float constant) {
      v_f_const = constant;
   }
   public void setOutputFile(PrintStream out) {
      this.out = out;
   }
   public void addElement(PhysicsElement e) {
      elements.add(e);
   }
   private void printSystemHeaders() {
      out.print("Time");
      for (PhysicsElement e: elements)  // print header
         out.print(",,"+e.getDescription());
      out.println();
   }  
   private void printSystemState(double t) {
      out.print(t);
      for (PhysicsElement e: elements)  
         out.print(",," + e.getState());  // print each element state
      out.println();
   }        

   public void simulate ( float delta_t, float samplingTime, float endTime){
      printSystemHeaders();
      for (PhysicsElement e: elements)
         e.setInitialState(g, v_f_const);
      for (double t=0, nextStateTime=0; t<endTime; t+=delta_t){
         if (t >= nextStateTime) {       // decide if it is time to print state 
            printSystemState(t);
            nextStateTime += samplingTime;
         } 
         for (PhysicsElement e: elements)  // for each element
            e.computeNextState(delta_t, g, v_f_const); // compute next state
         for (PhysicsElement e: elements)  // for all elements
            e.updateState();                // update its state
      }
   }
   private float g;
	private float v_f_const;
   private PrintStream out;
   private Vector<PhysicsElement> elements;
} 
